#ifndef __PAGING_H__
#define __PAGING_H__

#include <asm/types.h>

extern u8 *pbitmap;

extern u32 pages_free;
extern u32 pages_total;


void init_paging(void);

void init_pages(void);
u32 alloc_page(void);
void free_page(u32 addr);

/*
 * i don't care much about these macros
 * the system is not portable to other page sizes :)
 * so beware :)
 */

/* flags for page directory (PDE)/table entries (PTE) */
#define PG_PRESENT	0x01	/* is in memory - for swapping */
#define PG_WRITE	0x02	/* page could be written */
#define PG_USER		0x04	/* user mode stuff (CPL=3) */
#define PG_WT		0x08	/* write-through cache, else write-back cache */
#define PG_CACHE_OFF	0x10	/* disable caching of page or table */
#define PG_ACCESSED	0x20	/* page has been accessed, cleared by OS */
#define PG_DIRTY	0x40	/* has been written, cleared by OS */
#define PG_4MB		0x80	/* only for directory entries, in PTE = 0 */
#define PG_GLOBAL	0x100	/* not used - only for Pentiums */
#define PG_09		0x200	/* bits 9,10,11 are available */
#define PG_10		0x400	/* but if not PG_PRESENT, bits 1-31 available */
#define PG_11		0x800	/* rename them if they're used */

#define PAGE_SIZE		(1<<PAGE_SHIFT)		/* 4096 */
#define PAGE_SHIFT		12
#define ADDR2PAGE(addr)		((addr)>>PAGE_SHIFT)	/* addr/4096 */

/* don't forget to clear flags before using entries as pointer! */
#define PG_DIR_ENTRY(dir,addr)	( (u32*) dir[ (addr) >> 22 ] )
#define PG_TAB_ENTRY(tab,addr)	( (u32*) tab[ ( (addr) >> 12 ) & 0xfff ] )
/* returns correct physical address of page */
#define VIRT2PHYS(dir,n)	( PG_TAB_ENTRY( PG_DIR_ENTRY( dir, n ) & 0xFFFFF000, n ) & 0xFFFFF000 )

#endif /*__PAGING_H__*/
