/* unistd.h - standard symbolic constants and types */

#ifndef __UNISTD_H__
#define __UNISTD_H__

/*
     The <unistd.h> header defines miscellaneous symbolic constants and types,
     and declares miscellaneous functions. The actual values of the constants
     are unspecified except as shown. The contents of this header are shown
     below.
*/

/*
      Version Test Macros

     The following symbolic constants shall be defined:
   _POSIX_VERSION
          Integer value indicating version of IEEE Std 1003.1 (C-language
          binding) to which the implementation conforms. For implementations
          conforming to IEEE Std 1003.1-2001, the value shall be 200112L.
   _POSIX2_VERSION
          Integer value indicating version of the Shell and Utilities volume of
          IEEE Std 1003.1  to  which  the  implementation  conforms.  For
          implementations conforming to IEEE Std 1003.1-2001, the value shall
          be 200112L.
*/

#define	_POSIX_ADVISORY_INFO			-1
#define	_POSIX_ASYNCHRONOUS_IO			-1
#define	_POSIX_BARRIERS				-1

#undef	_POSIX_CHOWN_RESTRICTED

#define	_POSIX_CLOCK_SELECTION			-1
#define	_POSIX_CPUTIME				-1
#define	_POSIX_FSYNC				-1
#define	_POSIX_IPV6				-1

#define	_POSIX_JOB_CONTROL			-1
/*        The implementation supports job control. This symbol shall always be
          set to a value greater than zero. */

#define	_POSIX_MAPPED_FILES			-1
#define	_POSIX_MEMLOCK				-1
#define	_POSIX_MEMLOCK_RANGE			-1
#define	_POSIX_MEMORY_PROTECTION		-1
#define	_POSIX_MESSAGE_PASSING			-1
#define	_POSIX_MONOTONIC_CLOCK			-1

#undef	_POSIX_NO_TRUNC
/*        Pathname components longer than {NAME_MAX} generate an error. This
          symbol shall either be undefined or defined with a value other than
          -1.*/

#define	_POSIX_PRIORITIZED_IO			-1

/*this is easy*/
#define	_POSIX_PRIORITY_SCHEDULING		-1

#define	_POSIX_RAW_SOCKETS			-1
#define	_POSIX_READER_WRITER_LOCKS		-1
#define	_POSIX_REALTIME_SIGNALS			-1

#define	_POSIX_REGEXP				-1
/*        The implementation supports the Regular Expression Handling option.
          This symbol shall always be set to a value greater than zero.
*/

#define	_POSIX_SAVED_IDS			-1
/*        Each process has a saved set-user-ID and a saved set-group-ID. This
          symbol shall always be set to a value greater than zero.
*/

#define	_POSIX_SEMAPHORES			-1
#define	_POSIX_SHARED_MEMORY_OBJECTS		-1

#define	_POSIX_SHELL				-1
/*        The implementation supports the POSIX shell. This symbol shall always
          be set to a value greater than zero.
*/

#define	_POSIX_SPAWN				-1
#define	_POSIX_SPIN_LOCKS			-1
#define	_POSIX_SPORADIC_SERVER			-1
#define	_POSIX_SYNCHRONIZED_IO			-1
#define	_POSIX_THREAD_ATTR_STACKADDR		-1
#define	_POSIX_THREAD_ATTR_STACKSIZE		-1
#define	_POSIX_THREAD_CPUTIME			-1
#define	_POSIX_THREAD_PRIO_INHERIT		-1
#define	_POSIX_THREAD_PRIO_PROTECT		-1
#define	_POSIX_THREAD_PRIORITY_SCHEDULING	-1
#define	_POSIX_THREAD_PROCESS_SHARED		-1
#define	_POSIX_THREAD_SAFE_FUNCTIONS		-1
#define	_POSIX_THREAD_SPORADIC_SERVER		-1
#define	_POSIX_THREADS				-1
#define	_POSIX_TIMEOUTS				-1
#define	_POSIX_TIMERS				-1
#define	_POSIX_TRACE				-1
#define	_POSIX_TRACE_EVENT_FILTER		-1
#define	_POSIX_TRACE_INHERIT			-1
#define	_POSIX_TRACE_LOG			-1
#define	_POSIX_TYPED_MEMORY_OBJECTS		-1

#define	_POSIX_VDISABLE				-1
/*        This symbol shall be defined to be the value of a character that
          shall disable terminal special character handling as described in
          [96]<termios.h>. This symbol shall always be set to a value other
          than -1.*/

#define	_POSIX2_C_BIND				-1
/*        The implementation supports the C-Language Binding option. This
          symbol shall always have the value 200112L.*/

#define	_POSIX2_CHAR_TERM			-1
/*        The implementation supports at least one terminal type.*/
#define	_POSIX2_LOCALEDEF			-1

/*
   _POSIX_ASYNC_IO
          Asynchronous input or output operations may be performed for the
          associated file.
   _POSIX_PRIO_IO
          Prioritized input or output operations may be performed for the
          associated file.
   _POSIX_SYNC_IO
          Synchronized input or output operations may be performed for the
          associated file.
*/

#ifndef NULL
#define NULL	((void *) 0L)		/* Null pointer */
#endif

// in libc
//#define	F_OK	0	/* Test for existence of file. */
//#define	R_OK	4	/* Test for read permission. */
//#define	W_OK	2	/* Test for write permission. */
//#define	X_OK	1	/* Test for execute (search) permission. */
/*
     The constants F_OK, R_OK, W_OK, and X_OK and the expressions R_OK|W_OK,
     R_OK|X_OK, and R_OK|W_OK|X_OK shall all have distinct values.
*/


/* lseek() and fcntl() functions */
// in libc
//#define	SEEK_CUR	1	/* Set file offset to current plus offset. */
//#define	SEEK_END	2	/* Set file offset to EOF plus offset. */
//#define	SEEK_SET	3	/* Set file offset to offset. */


/* lockf() function */
#define	F_LOCK		1	/* Lock a section for exclusive use. */
#define	F_TEST		2	/* Test section for locks by other processes. */
#define	F_TLOCK		3	/* Test and lock a section for exclusive use. */
#define	F_ULOCK		4	/* Unlock locked sections. */

#define	STDERR_FILENO	2
#define	STDIN_FILENO	0
#define	STDOUT_FILENO	1

/*
      Type Definitions

     The size_t, ssize_t, uid_t, gid_t, off_t, pid_t, and useconds_t types
     shall be defined as described in [150]<sys/types.h>.

     The intptr_t type shall be defined as described in [151]<inttypes.h>.
*/

/*
      Declarations

     The following shall be declared as functions and may also be defined as
     macros. Function prototypes shall be provided.
*/
int          access(const char *, int);
unsigned     alarm(unsigned);
int          chdir(const char *);
int          chown(const char *, uid_t, gid_t);
int          close(int);
size_t       confstr(int, char *, size_t);

//^[[152]XSI] [Option Start]
char        *crypt(const char *, const char *);
char        *ctermid(char *);
// [Option End]
int          dup(int);

int          dup2(int, int);
//^[[153]XSI] [Option Start]
void         encrypt(char[64], int);
// [Option End]

int          execve(const char *, char *const [], char *const []);

void        _exit(int);
int          fchown(int, uid_t, gid_t);
//^[[154]XSI] [Option Start]
int          fchdir(int);
// [Option End]
//^[[155]SIO] [Option Start]
int          fdatasync(int);
// [Option End]
pid_t        fork(void);
long         fpathconf(int, int);
//^[[156]FSC] [Option Start]
int          fsync(int);
// [Option End]
int          ftruncate(int, off_t);
char        *getcwd(char *, size_t);
gid_t        getegid(void);
uid_t        geteuid(void);
gid_t        getgid(void);
int          getgroups(int, gid_t []);
//^[[157]XSI] [Option Start]
long         gethostid(void);
// [Option End]
int          gethostname(char *, size_t);
char        *getlogin(void);
int          getlogin_r(char *, size_t);

//int          getopt(int, char * const [], const char *);

//^[[158]XSI] [Option Start]
pid_t        getpgid(pid_t);
// [Option End]
pid_t        getpgrp(void);
pid_t        getpid(void);
pid_t        getppid(void);
//^[[159]XSI] [Option Start]
pid_t        getsid(pid_t);
// [Option End]
uid_t        getuid(void);
//^[[160]XSI] [Option Start]
char        *getwd(char *); (LEGACY )
// [Option End]
int          isatty(int);
//^[[161]XSI] [Option Start]
int          lchown(const char *, uid_t, gid_t);
// [Option End]
int          link(const char *, const char *);
//^[[162]XSI] [Option Start]
int          lockf(int, int, off_t);
// [Option End]
off_t        lseek(int, off_t, int);
//^[[163]XSI] [Option Start]
int          nice(int);
// [Option End]
long         pathconf(const char *, int);
int          pause(void);
int          pipe(int [2]);
//^[[164]XSI] [Option Start]
ssize_t      pread(int, void *, size_t, off_t);
ssize_t      pwrite(int, const void *, size_t, off_t);
// [Option End]
ssize_t      read(int, void *, size_t);
ssize_t      readlink(const char *restrict, char *restrict, size_t);
int          rmdir(const char *);
int          setegid(gid_t);
int          seteuid(uid_t);
int          setgid(gid_t);

int          setpgid(pid_t, pid_t);
//^[[165]XSI] [Option Start]
pid_t        setpgrp(void);
int          setregid(gid_t, gid_t);
int          setreuid(uid_t, uid_t);
// [Option End]
pid_t        setsid(void);
int          setuid(uid_t);
unsigned     sleep(unsigned);
//^[[166]XSI] [Option Start]
void         swab(const void *restrict, void *restrict, ssize_t);
// [Option End]
int          symlink(const char *, const char *);
//^[[167]XSI] [Option Start]
void         sync(void);
// [Option End]
long         sysconf(int);
pid_t        tcgetpgrp(int);
int          tcsetpgrp(int, pid_t);
//^[[168]XSI] [Option Start]
int          truncate(const char *, off_t);
// [Option End]
char        *ttyname(int);
int          ttyname_r(int, char *, size_t);
//^[[169]XSI] [Option Start]
useconds_t   ualarm(useconds_t, useconds_t);
// [Option End]
int          unlink(const char *);
//^[[170]XSI] [Option Start]
int          usleep(useconds_t);
pid_t        vfork(void);
// [Option End]
ssize_t      write(int, const void *, size_t);
/*
     Implementations may also include the [171]pthread_atfork() prototype as
     defined in [172]<pthread.h>.
*/

/*
     The following external variables shall be declared:
extern char  *optarg;
extern int    optind, opterr, optopt;
*/

/*
     Earlier versions of IEEE Std 1003.1 and of the Single UNIX Specification
     can be identified by the following macros:
   POSIX.1-1988 standard
          _POSIX_VERSION==198808L
   POSIX.1-1990 standard
          _POSIX_VERSION==199009L
   ISO POSIX-1:1996 standard
          _POSIX_VERSION==199506L
   Single UNIX Specification, Version 1
          _XOPEN_UNIX and _XOPEN_VERSION==4
   Single UNIX Specification, Version 2
          _XOPEN_UNIX and _XOPEN_VERSION==500

     IEEE Std 1003.1-2001 does not make any attempt to define application
     binary  interaction  with the underlying operating system. However,
     application developers may find it useful to query _SC_VERSION at runtime
     via  [173]sysconf() to determine whether the current version of the
     operating system supports the necessary functionality as in the following
     program fragment:
if (sysconf(_SC_VERSION) < 200112L) {
    fprintf(stderr, "POSIX.1-2001 system required, terminating \n");
    exit(1);
}
*/

/*
(FROM INFORMATIVE TEXT)

     The symbolic constants _XOPEN_REALTIME and _XOPEN_REALTIME_THREADS are
     added. _POSIX2_C_BIND, _XOPEN_ENH_I18N, and _XOPEN_SHM must now be set to
     a value other than -1 by a conforming implementation.

     The type of the argument to sbrk() is changed from int to intptr_t.

     _POSIX2_C_VERSION is removed.

     The brk() and sbrk() legacy functions are removed.
*/

#endif /*__UNISTD_H__*/
