#include <asm/printk.h>
#include <asm/selector.h>
#include <asm/types.h>
#include <asm/tss.h>
#include <asm/paging.h>
#include <string.h>

/*rename items*/
typedef struct tss_desc {
	u16	limit;		/* limit of the segment */
	u16	addr0_15;	/* address of segment (the TSS struct) */
	u8	addr16_23;
	u8	flags;		/* PDD0TTTT - present, DPL, 0, type */
	u8	flags0;		/* we use 0 */
	u8	addr24_31;
} tss_desc_t;

struct tss_desc *task_desc = (struct tss_desc *) ((u32) (&gdt + TASK_TSS));
struct tss_desc *kernel_desc = (struct tss_desc *) ((u32) (&gdt + KERNEL_TSS));

/*should not cross page boundary, align it*/
struct tss *task1;
u32 irq_yes;	/*testing*/

extern u32 *pd;		/* page directory */

/* kernel will never be swapped, don't care about alignment */
struct tss kernel_tss;


void init_tss()
{
	/* kernel TSS init */
	memset((void *) &kernel_tss, 0, sizeof (struct tss));

	kernel_desc->limit = sizeof (struct tss) - 1;
	kernel_desc->addr0_15 = (u16) ( (u32) &kernel_tss);
	kernel_desc->addr16_23 = (u8) (((u32) &kernel_tss) >> 16);
	kernel_desc->addr24_31 = (u8) (((u32) &kernel_tss) >> 24);
	/* present, DPL=0, available 386 TSS*/
	kernel_desc->flags = (1<<7) | 0x9;
	/* we're adjusting only these flags from now
	   (because of the dumb busy tasks)
	   rewrite it every time whe switching to kernel (it's 0x89)
	 */

	/* task TSS init, we don't have any address yet */
	task_desc->limit = sizeof (struct tss) - 1;
	task_desc->flags = (1<<7) | 0x9;
	/* these flags will be reloaded with 0x89 too, we do not allow user
	   mode tasks calling the TSS entry (so DPL has to be 0)
	*/

/**********************************************/
/*?*/	kernel_tss.ss0 = KERNEL_DATA;
kernel_tss.esp0 = 0x700000;
/**********************************************/

	kernel_tss.cr3 = pd;

	/*	eax,ecx,edx,ebx,esp,ebp,esi,edi,eip
		es,cs,ss,ds,fs,gs
		eflags
		
		these are stored in tss when task-switching
	*/

	/* initialize task register */
	__asm__ __volatile__( "ltr %%ax" : : "a" (KERNEL_TSS) );
	
/***********************************/
	/****TEST*****/
	extern void task();
	void init_ourtask(u32 task, u32 esp, u32 esp0, u32 *pd);
	/*alloc page dir, esp, esp0 in future*/
	init_ourtask((u32) &task, 0x300000-0x100, 0x5aff80, pd);
	irq_yes = 2;
}


/*****************************************************************************/

u32 init_ourtask(u32 task, u32 esp, u32 esp0, u32 *pd)
{
	extern u32 alloc_page();
	task1 = (struct tss *) alloc_page();

	if ((u32)task1 == 0)
		return 0;

	memset((void *) task1, 0, sizeof (struct tss));
	
/********ONLY WHEN SWITCHING!**/
	task_desc->addr0_15 = (u16) ((u32) task1);
	task_desc->addr16_23 = (u8) (((u32)task1)>>16);
	task_desc->addr24_31 = (u8) (((u32)task1)>>24);
/************************/

	task1->cr3 = pd;	/* task's page directory*/
	task1->eip = task;	/* task's entry point (from it's own mapping) */
	task1->esp = esp;	/* task's stack */

	/* RPL=3 */
	task1->cs = USER_CODE | 3;
	task1->ds = USER_DATA | 3;
	task1->es = USER_DATA | 3;
	task1->fs = USER_DATA | 3;
	task1->gs = USER_DATA | 3;
	task1->ss = USER_DATA | 3;

	task1->ss0 = KERNEL_DATA;
	task1->esp0 = esp0;		/* have to be allocated */


	/* get eflags register and make sure interrupts are enabled */
////TODO? i think int enable is enough, no other is needed - can set only that?
////set IOPL in EFLAGS!!!
	__asm__ __volatile__ ("pushfl ; popl %0" : "=g" (task1->eflags) );

	task1->eflags |= (1<<9);	/* interrupts enable*/
	task1->eflags &= ~(3<<12);	/* set IOPL to 0 */
	//TODO make IO bitmap

//////WE'RE USING KERNEL'S PAGE DIR, SO CHANGE AND INVALIDATE NOW!////////
extern u32 map(u32 *pd, u32 page, u32 addr, u32 flags);
map(pd,task&0xfffff000,task&0xfffff000,PG_PRESENT|PG_WRITE|PG_USER);
map(pd,esp&0xfffff000,esp&0xfffff000,PG_PRESENT|PG_WRITE|PG_USER);
	/* invalidate TLB */
	__asm__ __volatile__ (";\
		movl %%cr3, %%eax	;\
		movl %%eax, %%cr3"
		: : : /*clobbers*/ "eax");
//////////////////////////////////////////////////////////////////////////

	return 1;	/* success */
}
