#define SEL_ONLY	/* only selectors */
#include <asm/selector.h>

		.text
		.globl	load_gdt, gdt

	/* loads GDTR with kernel's Interrupt Descriptor Table */
	load_gdt:
		lgdt	gdt48

		/* reload segment registers with our own values */
		ljmp	$KERNEL_CODE, $reload_cs
reload_cs:
		movl	$KERNEL_DATA, %eax
		movl	%eax, %ds
		movl	%eax, %es
		movl	%eax, %ss

		xorl	%eax, %eax	/* we're not using those */
		movl	%eax, %fs
		movl	%eax, %gs

		lldt	%ax		/* clear */
		ret

/*
 *	our GDT stuff
 *	modify limit size if changed (and selector.h)
 */

		.data

		/*
			* intel docs say 8 for performance
			* linux comment says 16 is recommended by intel and
			  there are broken machines that need that
			it won't hurt, anyway
		*/
		.align 16
	gdt:
		.word	0, 0
		.byte	0, 0, 0, 0

		/*
		 * 32bit readable code segment
		 * privilege 0 (kernel)
		 * 4kB granularity (4GB)
		 * non-conforming (accessible only from privilege 0)
		 */
		.word	0xFFFF, 0
		.byte	0, 0x9A, 0xCF, 0

		/*
		 * 32bit writable data segment (expand-up)
		 * privilege 0 (kernel)
		 * 4kB granularity (4GB)
		 */
		.word	0xFFFF, 0
		.byte	0, 0x92, 0xCF, 0
		
		/*
		 * 32bit readable code segment
		 * privilege 3 (user)
		 * 4kB granularity (4GB)
		 * non-conforming (accessible from privilege 3,2,1,0)
		 */
		.word	0xFFFF, 0
		.byte	0, 0xFA, 0xCF, 0

		/*
		 * 32bit writable data segment (expand-up)
		 * privilege 3 (user)
		 * 4kB granularity (4GB)
		 */
		.word	0xFFFF, 0
		.byte	0, 0xF2, 0xCF, 0

		/* TSS descriptor for kernel */
		.quad	0
		/* TSS descriptor for applications */
		.quad	0

		/* -- 16 bit real mode CS --
		 * base = 0x00000000, limit 0x0FFFF (1 B Granularity), present
		 * type = 16 bit code execute/read only/conforming, DPL = 0
		 */
//		.word	0xFFFF, 0
//		.byte	0, 0x9E, 0, 0

		/* -- 16 bit real mode DS --
		 * base = 0x00000000, limit 0x0FFFF (1 B Granularity), present
		 * type = 16 bit data read/write, DPL = 0
		 */
//		.word	0xFFFF, 0
//		.byte	0, 0x92, 0, 0

	gdt48:
		.word	(8*7)-1			/* limit (8n-1) */
		.long	gdt			/* address of the table */
