#include <kernel.h>
#include <asm/printk.h>
#include <asm/types.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/selector.h>
#include <asm/idt.h>
#include <asm/pit.h>
#include <asm/paging.h>

/* use this with critical init functions (not vga, lgdt, lidt) */
#define INIT(proc,str)	{ puts(str); proc; puts(" OK\n");}

#include <multiboot.h>
multiboot_info_t *mbi;

/* Multiboot compliant kernel entry point */
void main(u32 magic, u32 addr)
{
	mbi = (multiboot_info_t *) addr;	/* save for later use */

	init_vga();		/* set up stuff for puts() */
	
extern void multiboot_info(u32 magic, u32 addr);
//	multiboot_info(magic, addr);

extern void set_overscan(u8 color);
//	set_overscan(4);/*red*/

	/* remap IRQ's from PIC1 to 0x20-0x27, from PIC2 to 0x28-0x2F */
	remap_pics(0x20, 0x28);
	/* mask all unused irq's, enable when loading IDT with real handler */
	outb(0xFF, 0x21);		/* mask IRQ0-7 */
	outb(0xFF, 0xA1);		/* mask IRQ8-F */

	load_gdt();
	init_idt();
	init_pit();			/* set up before sti, 100Hz */

	INIT(sti(), "STI")
	INIT(init_paging(), "PG")
	
	printk("Memory: %d kB (%dkB used by kernel & reserved areas)\n",pages_total*4, (pages_total-pages_free)*4);

/*****************************************************************************/
//__asm__("xorw %ax,%ax ; divb %ah,%al");	/* 0/0 */
__asm__("int $3");				/*breakpoint*/
//__asm__("ud2");				/*invalid opcode :)*/
//__asm__("movl $0x50,%eax ; movl %eax,%ds");	/*general protection - error code 0x50*/

extern void init_tss();
init_tss();
			extern void testcmos();
			testcmos();
			for(;;);
extern u32 task_cnt;
for (int i=0; ; )
	if (i<ticks) {
		i += 100;	/*why 1000 ticks = 2s ??? should be 10*/
		printk("%d\n",task_cnt);
	}
/*****************************************************************************/

	for (;;)
		;

	return;					/* never get here */
}
