/* completely based on GRUB's example kernel */

#include <multiboot.h>
#include <asm/types.h>
#include <asm/printk.h>

void multiboot_info(u32 magic, u32 addr)
{
	multiboot_info_t *mbi;	/* structure passed by bootloader */

	/* Am I booted by a Multiboot-compliant boot loader?  */
	if (magic != MULTIBOOT_BOOTLOADER_MAGIC) {
		printk("Invalid magic number: 0x%x\nHalt", magic);
		for (;;)
			;
	}

	/* Set MBI to the address of the Multiboot information structure.  */
	mbi = (multiboot_info_t *) addr;

	/* Print out the flags.  */
	printk("flags = 0x%x\n", (u32) mbi->flags);
	/* Are mem_* valid?  */
	if (CHECK_FLAG(mbi->flags, 0))
		printk("mem_lower = %ukB, mem_upper = %ukB\n",
		       (u32) mbi->mem_lower, (u32) mbi->mem_upper);
	/* Is boot_device valid?  */
	if (CHECK_FLAG(mbi->flags, 1))
		printk("boot_device = 0x%x\n", (u32) mbi->boot_device);
	/* Is the command line passed?  */
	if (CHECK_FLAG(mbi->flags, 2))
		printk("cmdline = %s\n", (char *) mbi->cmdline);
	/* Are mods_* valid?  */
	if (CHECK_FLAG(mbi->flags, 3)) {
		module_t *mod;
		int i;

		printk("mods_count = %d, mods_addr = 0x%x\n",
		       (int) mbi->mods_count, (int) mbi->mods_addr);
		for (i = 0, mod = (module_t *) mbi->mods_addr;
		     i < mbi->mods_count; i++, mod++)
			printk(" mod_start = 0x%x, mod_end = 0x%x, string = %s\n", (u32) mod->mod_start, (u32) mod->mod_end, (char *) mod->string);
	}
	/* Bits 4 and 5 are mutually exclusive!  */
	if (CHECK_FLAG(mbi->flags, 4) && CHECK_FLAG(mbi->flags, 5)) {
		printk("Both bits 4 and 5 are set.\nHalt");
		for (;;)
			;
	}
	/* Is the symbol table of a.out valid?  */
	if (CHECK_FLAG(mbi->flags, 4)) {
		aout_symbol_table_t *aout_sym = &(mbi->u.aout_sym);

		printk("aout_symbol_table: tabsize = 0x%0x, "
		       "strsize = 0x%x, addr = 0x%x\n",
		       (u32) aout_sym->tabsize,
		       (u32) aout_sym->strsize, (u32) aout_sym->addr);
	}
	/* Is the section header table of ELF valid?  */
	if (CHECK_FLAG(mbi->flags, 5)) {
		elf_section_header_table_t *elf_sec = &(mbi->u.elf_sec);

		printk("elf_sec: num = %u, size = 0x%x,"
		       " addr = 0x%x, shndx = 0x%x\n",
		       (u32) elf_sec->num, (u32) elf_sec->size,
		       (u32) elf_sec->addr, (u32) elf_sec->shndx);
	}
	/* Are mmap_* valid?  */
	if (CHECK_FLAG(mbi->flags, 6)) {
		memory_map_t *mmap;

		printk("mmap_addr = 0x%x, mmap_length = 0x%x\n",
		       (u32) mbi->mmap_addr, (u32) mbi->mmap_length);
		for (mmap = (memory_map_t *) mbi->mmap_addr;
		     (u32) mmap < mbi->mmap_addr + mbi->mmap_length;
		     mmap =
		     (memory_map_t *) ((u32) mmap + mmap->size +
				       sizeof(mmap->size)))
			printk(" size = 0x%x, base_addr = 0x%x%x,"
			       " length = 0x%x%x, type = 0x%x\n",
			       (u32) mmap->size,
			       (u32) mmap->base_addr_high,
			       (u32) mmap->base_addr_low,
			       (u32) mmap->length_high,
			       (u32) mmap->length_low, (u32) mmap->type);
	}
}
