/*
	Programmable Interrupt Controller 8259A and
	Non-Maskable Interrupts enable/disable from OS FAQ
	(corrected and adjusted for my use)
	
	IRQ masking (not tested but (according to what i know) it should work)

	(NMI not tested, i checked newer version from OS FAQ - it's correct)
 */
 
#include <asm/io.h>
#include <asm/irq.h>

/* masks IRQ */
inline void mask_irq(u8 irq)
{
	if (irq & 0x7) {	/* <=7 */
		outb(inb_p(0x21)|(1<<irq),0x21);
		return;
	}

	if (irq & 0x8) {	/* >=8 */
		outb(inb_p(0xA1)|(1<<irq),0xA1);
		return;
	}
}

/* unmasks IRQ */
inline void unmask_irq(u8 irq)
{
	if (irq & 0x7) {	/* <=7 */
		outb(inb_p(0x21)&~((u8) (1<<irq)),0x21);
		return;
	}

	if (irq & 0x8) {	/* >=8 */
		outb(inb_p(0xA1)&~((u8) (1<<irq)),0xA1);
		return;
	}
}

void remap_pics(u32 pic1, u32 pic2)
{
	u8 a1, a2;

	a1 = inb(PIC1_DATA);
	a2 = inb(PIC2_DATA);

	outb_p(ICW1_INIT+ICW1_ICW4, PIC1_COMMAND);
	outb_p(ICW1_INIT+ICW1_ICW4, PIC2_COMMAND);
	outb_p(pic1, PIC1_DATA);
	outb_p(pic2, PIC2_DATA);
	outb_p(4, PIC1_DATA);
	outb_p(2, PIC2_DATA);

	outb_p(ICW4_8086, PIC1_DATA);
	outb_p(ICW4_8086, PIC2_DATA);

	outb(a1, PIC1_DATA);
	outb(a2, PIC2_DATA);
}


/* enable the NMI */
void nmi_enable(void)
{
	outb(inb(0x70)&0x7F, 0x70);
}

/* disable the NMI */
void nmi_disable(void)
{
//???//v linuxe davaju do 0x70 0x80 a nie inb()|0x80 (startup.S)
	outb(inb(0x70)|0x80, 0x70);
}
