#include <fancy/vga.h>

#include <asm/printk.h>
#include <asm/types.h>
#include <asm/io.h>

#define	PRINTK_BUFFER_SIZE	512	/* enough */
#define TABSIZE	8			/* for puts() */

struct {
	u32	x, y;
	u16	attr;		/* this is mask, must be (ATTRIBUTES<<8) | 0x00
				   for more details in docs/colors.txt */
} scr;


/*
	scrolls screen by 1 line
	( dest: SCREEN, src: SCREEN+(COLS*2), size: (LINES-1)*COLS * 2 )
	then clear last line at (dest+size)
*/
static void scroll(void)
{
#	define SIZE	( (LINES-1)*COLS * 2 )
#	define DEST	SCREEN
#	define SRC	SCREEN+(COLS*2)
/*
	for (int i=0; i<SIZE; i+=4)
		*((u32 *) (DEST + i)) = *((u32 *) (SRC + i));
*/
	int dummy;
	/* i386 assembler version */
	__asm__ __volatile__ ("cld ; rep ; movsl"
				: "=c" (dummy), "=D" (dummy), "=S" (dummy)
				: "c" (SIZE>>2), "D" (DEST), "S" (SRC)
				: /*clobbers*/ "memory" );

	u32 longmask = ((scr.attr | ' ') << 16) | scr.attr | ' ';
/*
	for (int i=0; i<(COLS/2); i+=4)
		*(u32 *) (SCREEN+SIZE + i) = longmask;
*/
	/* assembler once again */
	__asm__ __volatile__ ("cld ; rep ; stosl"
				: "=c" (dummy), "=D" (dummy)
				: "a" (longmask), "c" (COLS>>1), "D" (SCREEN+SIZE) 
				: /*clobbers*/ "memory" );
}

/*
 *	works fine. no bugs witnessed :)
 *	'\n' - newline (carriage return + line feed)
 *	'\r' - carriage return
 *	'\t' - tab (to next TABSIZE boundary)
 *	'\b' - backspace
 *
 *	TODO? i don't know if i should do '\f' form feed (and how in here it's blank line)
 */
void puts(const char *str)
{
	u8 c;
	u32 x, y;

	u32 tab;		/* for calculating next tab distance */
	
	x = scr.x;
	y = scr.y;

	while ( (c = *str++) != '\0' )
		switch (c) {
			case '\n':
				if ( ++y == LINES ) {	/* on last line */
					scroll();
					--y;
				}
				/* merge with '\r' */
			case '\r':
				x = 0;
				break;
			case '\t':
				tab = TABSIZE - ( x % TABSIZE );
				for ( int i=0; i<tab; i++ )
					*((u16 *) (SCREEN+((x+(y*COLS)+i)<<1))) = scr.attr | ' ';
				x += tab;
				if ( x >= COLS ) {
					x %= COLS;
					if ( ++y == LINES ) {
						scroll();
						--y;
					}
				}
				break;
			case '\b':
				if ( !x && !y )	/* left upper corner */
					break;
				if ( --x < 0 )
					x = COLS - 1;
				*((u16 *) (SCREEN+((x+(y*COLS))<<1))) = scr.attr | ' ';
				break;
			default:
				*((u16 *) (SCREEN+((x+(y*COLS))<<1))) = scr.attr | c;
				if ( ++x == COLS ) {
					x = 0;
					if ( ++y == LINES ) {
						scroll();
						--y;
					}
				}
				break;
		}

	scr.x = x;
	scr.y = y;
	
	/* update cursor position */
	u32 pos = (x + COLS * y) * 2;
//__asm__ ("cli");
	outb_p(14, VGA_INDEX);
	outb_p(0xff & (pos >> 9), VGA_DATA);
	outb_p(15, VGA_INDEX);
	outb_p(0xff & (pos >> 1), VGA_DATA);
//__asm__ ("sti");
////	cli+sti?
}


/* 
 * Prints formatted string with variable arguments
 *
 * from vsnprintf():
 * The return value is the number of characters which would
 * be generated for the given input, excluding the trailing
 * '\0', as per ISO C99. If you want to have the exact
 * number of characters written into @buf as return value
 * (not including the trailing '\0'), use vscnprintf. If the
 * return is greater than or equal to @size, the resulting
 * string is truncated.
 */
#include <lib-linux/types.h>	/*size_t for vsnprintf()*/
#include <stdarg.h>
extern int vsnprintf(char *buf, size_t size, const char *fmt, va_list args);
int printk(const char *fmt, ...)
{
	va_list args;
	int i;

	char buffer[PRINTK_BUFFER_SIZE];

	va_start(args, fmt);
	i = vsnprintf(buffer, PRINTK_BUFFER_SIZE, fmt, args);
	va_end(args);

	puts(buffer);
	return i;
}

/*
	init this stuff
	we won't complicate this (assume VGA color 80x25 (mode 0x03)):
	get cursor position and set default attributes
 */
void init_vga(void)
{
	/* get cursor position, continue from there - don't clear the screen */
//	u16 pos= *((u16 *) 0x450);		/* from BIOS */
//	scr.x = pos & 0xff;
//	scr.y = pos >> 8;
	
/*
	or from the hardware (i dunno where this originated):
	<use outb_p? but this works on my PC>
*/
		u16 pos;
		outb_p(14, VGA_INDEX);
		pos = inb_p(VGA_DATA) << 8;
		outb_p(15, VGA_INDEX);
		pos |= inb(VGA_DATA);
		
		scr.x = pos % COLS;
		scr.y = pos / COLS;
/**/
	
	/* set the default attributes */
	scr.attr = (BG_BLACK | FG_GRAY) << 8;
}
