#include <asm/types.h>
#include <asm/paging.h>
#include <string.h>

/*
	maps the given physical page in page directory
	if the page table is not present, try to allocate one
	u32 page must be given a page address (on boundary)
*/
u32 map(u32 *pd, u32 page, u32 addr, u32 flags)
{
	/* get the table address */
	u32 *pt = (u32 *) (pd[addr>>22] & 0xFFFFF000);

	if (! (u32) pt) {	/* don't have page table allocated */
		u32 pde;
		if ((pde = alloc_page()) != 0) {
			memset((void *) pde, 0, 4096);
			/* set directory entry with table address and flags */
			pd[addr>>22] = pde | PG_WRITE | PG_USER | PG_PRESENT;
			pt = (u32 *) pde;
		} else
			return 0;	/* can't allocate page table */
	}

	pt[(addr >> 12) & 0x3ff] = page | flags;
	return 1;
}

/* just unmaps the linear address from page directory */
u32 unmap(u32 *pd, u32 addr)
{
	/* get the table address */
	u32 *pt = (u32 *) (pd[addr>>22] & 0xFFFFF000);
	if ((u32) pt)	/* no page table */
		return 0;
	
	pt[(addr >> 12) & 0x3ff] = 0;	/* clear the entry */
	return 1;
}
