#include <asm/types.h>
#include <kernel.h>
#include <string.h>
#include <multiboot.h>

u8 *pbitmap;

u32 pages_free = 0;
u32 pages_total = 0;

extern void _end, origin;

/*
	finds free page in bitmap, sets as used and returns address
	(searches from the top down - high memory first)
 */
u32 alloc_page(void)
{
	if (pages_free)
		--pages_free;

	for (int i=(pages_total-1)>>3; i >= 0; i--)
		if (pbitmap[i] != 0xff)			/* some free there */
			for (int a=(1<<7), j=7; a>0; a>>=1, j--)	/* higher first */
				if ( !(pbitmap[i]&a) ) {	/* not used */
					pbitmap[i] |= a;	/* mark it */
					return (((i<<3)+j)<<12);
				}
	/* her should come swapping? */
	return 0;	/* couldn't find any free pages */
}

/*
	deallocates page (may be passed as address of any byte from the page)
 */
void free_page(u32 addr)
{
	u8 tmp = pbitmap[addr>>15];
	
	if ( tmp != (pbitmap[addr>>15] &= (u8) ~(1<<((addr>>12)&0x7))))	
		pages_free++;

	return;
}

/*
	after successful execution, you may allocate pages
	here's the one and only allocation without alloc_page()
 */
void init_pages(void)
{
	/* page bitmap at the end of kernel, page aligned */
	if ( ((u32) &_end) & 0xfff )	/* not on page boundary */
		pbitmap = (u8*) ((u32) &_end) + (4096 - (((u32) &_end) & 4095));
	else
		pbitmap = (u8*) ((u32) &_end);

	/* clear the extreme case of bitmap - 128kB (for 4GB) */
	memset((void *) pbitmap, 0, 128*1024);


///TODO?
//	maybe add reserved areas too. mark them as used!!!
//	holes between usable memory would be fatal!!!
//	in this respect unsing mem_lower is safer than mmap
///
	/* add the upper memory pages */
//	if (0) {
	if (CHECK_FLAG(mbi->flags, 6)) {	/* are mmap_* valid?  */
		memory_map_t *mmap = (memory_map_t *) mbi->mmap_addr;
#		define MMAP_END		(mbi->mmap_addr + mbi->mmap_length)
#		define MMAP_NEXT	((u32) mmap + mmap->size + sizeof(mmap->size))
		u32 next = (1<<20);	/* first at 1MB */
		for ( ; (u32) mmap < MMAP_END; mmap = (memory_map_t *) MMAP_NEXT)
			/* usable & not low memory */
			if ( mmap->type == 0x01 && mmap->base_addr_low >= (1<<20)) {
				if (next != mmap->base_addr_low) {
					printk("hole detected, can't use memory at 0x%08x and above\n", mmap->base_addr_low);
					break;	/* can't have holes in memory */
				}
				pages_total += mmap->length_low >> 12;
				next = mmap->base_addr_low + mmap->length_low;
			}
	}
	else if (CHECK_FLAG(mbi->flags, 0))	/* are mem_* valid? */
		pages_total += (u32) mbi->mem_upper >> 2;	/* in kB */
	else {
		printk("PANIC! No memory info provided by loader...\nHalt");
		cli();	/* just to be sure */
		for (;;)
			;
	}

	pages_free = pages_total;

	/* 1 MB - lower memory, but don't count them as free */
	pages_total += (1 << 20) >> 12;

	/* always set 0 - 1MB as used */
	for (int i=0; i<(1<<20)>>15; i++) {
		pbitmap[i] = 0xff;
	}

	/* allocate kernel space */
	for (int i=((u32) &origin) >> 12; i<=((u32) &_end >> 12); i++) {
		pbitmap[i>>3] |= 1 << (i & 0x7);
		--pages_free;
	}

	/* alloc the bitmap too */
	for (int i=0; i<=(pages_total-1)>>15; i++)
		pbitmap[(((u32) pbitmap)>>15) + (i>>3)] |= 1 << (i & 0x7);
	pages_free -= ((pages_total-1)>>15) + 1;

	/*
		this marks non-existing pages in the last page bitmap entry
		as used. allocator must not use them (and it searches from top)
		(i think that's not possible, but you never know :))
	*/
	if (pages_total & 0x7) {
		/* mark non-existing as used */
		pbitmap[(pages_total-1)>>3] |= 0xff << (pages_total & 0x7);
		printk("This machine has %d pages. %d faked...\n", ~(pages_total-1) & 0x7);
	}
}
